from pyjuhelpers.basicconfig import  Config, AttrDict

import yaml


def test_dot_access():
    c = Config("configa.yaml")
    assert c.anycsv is not None
    assert c.anyCSV is not None

    assert c.anycsv.sniff_lines == 50
    assert c.anyCSV.sniff_lines == 50

def test_updates():
    c = Config("configa.yaml")

    assert c.anycsv.sniff_lines == 50

    c.merge("configb.yaml")
    assert c.anycsv.sniff_lines == 30




import logging.config

from pyjuhelpers.timer import Timer

logging.getLogger(__name__).addHandler(logging.NullHandler())


defaultConf={
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {
            'format': '%(levelname)8s %(message)s'
        },
        'debug': {
            'format': '[%(process)d] [%(asctime)s] [%(relativeCreated)18s][%(levelname)6s] [%(message)s]  [%(filename)s:%(lineno)s]'
        }
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'formatter': 'standard',
            'class': 'logging.StreamHandler',
        },
        'console_debug': {
            'level': 'DEBUG',
            'formatter': 'debug',
            'class': 'logging.StreamHandler',
        },
        'file': {
            'class': 'logging.handlers.RotatingFileHandler',
            'level': 'DEBUG',
            'formatter': 'debug',
            'filename': 'debug.log',
            'mode': 'a',
            'maxBytes': 103116800, #50MB
            'backupCount': 5,
            "encoding": "utf8"
        },
    },
    'root': {
            'handlers': ['console'],
            'level': 'WARNING',

    }

}
import copy

infoConf=copy.deepcopy(defaultConf)
infoConf['root']['handlers']=['console']
infoConf['root']['level']='INFO'


debugConf=copy.deepcopy(defaultConf)
debugConf['root']['handlers']=['console_debug']
debugConf['root']['level']='DEBUG'


fileConf=copy.deepcopy(defaultConf)
fileConf['root']['handlers']=['console','file']
fileConf['root']['level']='DEBUG'
fileConf['handlers']['console']['level']='WARNING'

import structlog

structlog.configure(#
    processors=[
        #structlog.stdlib.filter_by_level,
        #structlog.stdlib.add_logger_name,
        #structlog.stdlib.add_log_level,

        structlog.stdlib.PositionalArgumentsFormatter(),
        #structlog.processors.TimeStamper(fmt="%Y-%m-%d %H:%M:%S.%f"),
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
        structlog.processors.UnicodeDecoder(),

        structlog.dev.ConsoleRenderer(colors=False)

        ],
    context_class=dict,
    logger_factory=structlog.stdlib.LoggerFactory(),
    wrapper_class=structlog.stdlib.BoundLogger,
    cache_logger_on_first_use=True,
)
#print(structlog.get_config())

import time


def get_func_param(decorated_function, dec_fn_args, dec_fn_kwargs):
    arg_names = decorated_function.__code__.co_varnames[:decorated_function.__code__.co_argcount]
    args = dec_fn_args[:len(arg_names)]
    defaults = decorated_function.__defaults__ or ()
    args = args + defaults[len(defaults) - (decorated_function.__code__.co_argcount - len(args)):]
    params = list(zip(arg_names, args))
    args = dec_fn_args[len(arg_names):]
    if args:
        params.append(('args', args))

    param_dict = {}
    for p in params:
        param_dict[p[0]] = repr(p[1])
    if dec_fn_kwargs:
        for k, v in dec_fn_kwargs.items():
            param_dict[k] = repr(v)

    return param_dict

def log_func_detail(logger, log_time=False, time_key=None, level=logging.DEBUG):
    def log_function_entry_and_exit(decorated_function):
        """
        Function decorator logging entry + exit and parameters of functions.

        Entry and exit as logging.info, parameters as logging.DEBUG.
        """
        from functools import wraps


        @wraps(decorated_function)
        def wrapper(*dec_fn_args, **dec_fn_kwargs):

            func_name = decorated_function.__name__
            param_dict = get_func_param(decorated_function, dec_fn_args,dec_fn_kwargs)


            logger.log(level,"{}".format(func_name), **param_dict)
            # Execute wrapped (decorated) function:

            start=time.time()
            if log_time or time_key:
                _time_key = time_key if time_key else func_name

                with Timer(key=_time_key, store=True) as t:
                    out = decorated_function(*dec_fn_args, **dec_fn_kwargs)
            else:
                out = decorated_function(*dec_fn_args, **dec_fn_kwargs)
            end = time.time()
            if log_time:
                param_dict['elapsed']=((end-start)*1000)
                logger.log(level,"DONE {}".format(func_name), **param_dict)

            return out
        return wrapper
    return log_function_entry_and_exit


#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals



from io import StringIO
from operator import itemgetter
from statistics import mean
import numpy
__author__ = 'jumbrich'
import collections
import functools
import time


class Timer(object):
    GLOBAL_VERBOSE=True
    measures={}

    def __init__(self,  key=None, verbose=False, store=True):
        self.verbose = verbose
        self.key=key
        self.store=store

    def __enter__(self):
        self.start = time.time()
        return self

    def __exit__(self, *args):
        end = time.time()
        secs = end - self.start
        msecs = secs * 1000  # millisecs
        if self.verbose and Timer.GLOBAL_VERBOSE:
            print('(%s) elapsed time: %f ms' % (self.key,msecs))
        if self.key and self.store:
            if self.key not in self.__class__.measures:
                self.__class__.measures[self.key]=[]
            if msecs>=0:
                self.__class__.measures[self.key].append(msecs)

    @classmethod
    def printStats(cls, key=None, keys=None, header=True, sort_key=[7]):

        h1 = "\n {:^30} {:^5} {:^10} {:^10} {:^10} {:^10} {:^10} {:^10}".format("method",'count','min','q25','avg','q75','max','sum')
        l_s= len(h1)
        l="\n {:-^"+str(l_s)+"}"
        l = l.format("")
        s=""
        data=cls.getStatsAsLists()
        data=sorted(data, key=itemgetter(*sort_key), reverse=True)
        for row in data:
            m=row[0]
            if (key is None and  keys is None) or (key and m== key) or (keys and m in keys):
                _s_="\n {:>30} {:>5} {:>10.3f} {:>10.3f} {:>10.3f} {:>10.3f} {:>10.3f} {:>10.3f}"\
                    .format(m,*row[1:])
                s+=_s_

        h="\n{:-^"+str(l_s)+"}"
        h=h.format(" Timing stats ")
        if header:
            return h+h1+l+s+l
        else:
            return h1+s

    @classmethod
    def getStatsAsLists(cls):
        data=[]
        for m,v in cls.measures.items():
            data.append([m,len(v), min(v), numpy.percentile(v,0.25), mean(v), numpy.percentile(v,0.75),max(v), sum(v)])
        return data


    @classmethod
    def getStats(cls):
        stats={}
        h=["method", 'count', 'min', 'q25', 'avg', 'q75', 'max', 'sum']
        for row in cls.getStatsAsLists():
            stats[row[0]]={h[i]:row[i] for i in range(1, len(h))}
        return stats

    @classmethod
    def to_csv(cls, filename=None):
        import csv
        data = StringIO()
        csvw = csv.writer(data)

        csvw.writerow(["method",'count','min','q25','avg','q75','max','sum'])
        for row in cls.getStatsAsLists():
            csvw.writerow(row)
        if filename:
            with open(filename,'w') as f:
                f.write(data)
        data.seek(0)
        return data.read()


def timer(key=None, verbose=False, *func_or_func_args):
    def wrapped_f(f):

        @functools.wraps(f)
        def wrapped(*args, **kwargs):

            func_name = f.__name__
            #if key is None:
            #    key = func_name
            if key:
                with Timer(key=key, verbose=verbose) as t:
                    out = f(*args, **kwargs)
            else:
                with Timer(key=func_name, verbose=verbose) as t:
                    out = f(*args, **kwargs)
            return out
        return wrapped
    if (len(func_or_func_args) == 1
            and isinstance(func_or_func_args[0], collections.Callable)):
        return wrapped_f(func_or_func_args[0])
    else:
        return wrapped_f



from pyjuhelpers.basicconfig import Map, Config, AttrDict

import yaml




data = yaml.load(open(,"r"))

c = Config()
t = AttrDict(data['anycsv'])


data['anycsv']['sniff_lines'] = 30

print(t.sniff_lines)
t.update(data['anycsv'])
print(t.sniff_lines)


print(c.Anycsv.sniff_lines)

c.merge(init=data)
print(c.Anycsv.sniff_lines)




import yaml



class AttrDict(dict):

    @classmethod
    def _k(cls, key):
        return key.lower() if isinstance(key, str) else key

    def __init__(self, *args, **kwargs):
        super(AttrDict, self).__init__(*args, **kwargs)
        self._convert_keys()

    def __getitem__(self, key):
        return super(AttrDict, self).__getitem__(self.__class__._k(key))

    def __setitem__(self, key, value):
        super(AttrDict, self).__setitem__(self.__class__._k(key), value)

    def __delitem__(self, key):
        return super(AttrDict, self).__delitem__(self.__class__._k(key))

    def __contains__(self, key):
        return super(AttrDict, self).__contains__(self.__class__._k(key))

    def has_key(self, key):
        return super(AttrDict, self).has_key(self.__class__._k(key))

    def pop(self, key, *args, **kwargs):
        return super(AttrDict, self).pop(self.__class__._k(key), *args, **kwargs)

    def get(self, key, *args, **kwargs):
        return super(AttrDict, self).get(self.__class__._k(key), *args, **kwargs)

    def setdefault(self, key, *args, **kwargs):
        return super(AttrDict, self).setdefault(self.__class__._k(key), *args, **kwargs)

    def update(self, E={}, **F):
        super(AttrDict, self).update(self.__class__(E))
        super(AttrDict, self).update(self.__class__(**F))

    def _convert_keys(self):
        for k in list(self.keys()):
            v = super(AttrDict, self).pop(k)
            self.__setitem__(k, v)

    def __getattr__(self, attr):
        attr = attr.lower()
        return self[attr]

    def __setattr__(self, attr, value):
        attr= attr.lower()
        self[attr] = value



class Config(AttrDict):
    """

    """
    def __init__(self, config_file=None, init=None):
        if config_file:
            data = yaml.load(open(config_file, "r"))
            for k, v in data.items():
                k = k.lower()
                self[k] = AttrDict(v)

        if init:
            for k,v in init.items():
                k=k.lower()
                self[k] = AttrDict(v)

    def merge(self, config_file=None, init=None):
        if config_file:
            data = yaml.load(open(config_file, "r"))
            self._update(data)
        if init:
            self._update(init)

    def _update(self,data):

        for k, v in data.items():
            k = k.lower()
            if k in self:
                self[k].update(v)
            else:
                self[k] = AttrDict(v)

